"""
AI Logger - A module for AI-friendly JSON logging.

This package provides structured logging in a format that's
easily parsable by AI systems, with rich context.
"""

# Re-export key functionality for direct imports
from ai_logger.src import (
    init_ai_logger, 
    AILogger, 
    auto_wrap, 
    auto_wrap_class,
    CONFIG,
    reload_config,
    get_db_config,
    DatabaseConfig,
    LoggingConfig,
    AppConfig
)

# Export all important symbols for direct import from ai_logger
__all__ = [
    'init_ai_logger',
    'AILogger',
    'auto_wrap',
    'auto_wrap_class',
    'CONFIG',
    'reload_config',
    'get_db_config',
    'DatabaseConfig',
    'LoggingConfig',
    'AppConfig',
]


from __future__ import annotations
import os
from pathlib import Path
from typing import Dict
from pydantic import ValidationError
import json
import sys
import logging
from .models import AppConfig

def load_config(config_path: str = None) -> AppConfig:
    """
    Load and validate configuration from config file.
    
    Args:
        config_path: Optional path to a custom config file. If None, the default config is used.
    
    Returns:
        AppConfig: Loaded and validated configuration
    """
    # Avoid circular import
    from ai_logger.src.user_interface.uiux import ui
    
    # Check environment variable for config path
    env_config_path = os.environ.get('AI_LOGGER_CONFIG')
    
    # Priority: 1. Function argument, 2. Environment variable, 3. Default location
    if config_path:
        config_path = Path(config_path)
    elif env_config_path:
        config_path = Path(env_config_path)
    else:
        # Default config file location
        config_path = Path(__file__).parent / "ai_logger.json"

    try:
        # If config doesn't exist at specified path, try to use default
        if not config_path.exists():
            default_path = Path(__file__).parent / "ai_logger.json"
            if default_path.exists() and config_path != default_path:
                print(f"Warning: Config file not found at {config_path}, using default config")
                config_path = default_path
            else:
                ui.print(f"Config file not found at path: {config_path}", style="danger")
                # Instead of exiting, return default configuration
                return AppConfig()
        
        with open(config_path) as f:
            config_data = json.load(f)
            
        # Validate configuration
        return AppConfig(**config_data)
    except json.JSONDecodeError as e:
        ui.print(f"Invalid JSON in config file: {e}", style="danger")
        # Return default config instead of exiting
        return AppConfig()
    except ValidationError as e:
        ui.print(f"Invalid configuration: {e}", style="danger")
        return AppConfig()
    except Exception as e:
        ui.print(f"Error loading configuration: {e}", style="danger")
        return AppConfig()

# Load the configuration initially with default settings
CONFIG = load_config()

# Export common configuration values for backward compatibility
VERBOSE = CONFIG.verbose
USE_DB = CONFIG.database.use_db

def reload_config(config_path: str = None) -> AppConfig:
    """
    Reload the configuration from the specified path and update global CONFIG
    
    Args:
        config_path: Path to the configuration file
        
    Returns:
        The loaded AppConfig
    """
    global CONFIG, VERBOSE, USE_DB
    
    # Reload the configuration
    CONFIG = load_config(config_path)
    
    # Update the exported values
    VERBOSE = CONFIG.verbose
    USE_DB = CONFIG.database.use_db
    
    return CONFIG

def get_db_config() -> Dict:
    """Get database configuration as a dictionary."""
    return CONFIG.database.model_dump()

def get_log_level() -> int:
    """Get log level as a logging constant."""
    level_map = {
        "DEBUG": logging.DEBUG,
        "INFO": logging.INFO,
        "WARNING": logging.WARNING,
        "ERROR": logging.ERROR,
        "CRITICAL": logging.CRITICAL
    }
    return level_map.get(CONFIG.logging.level.upper(), logging.INFO)

def load_env_from_json(file_path):
    """Load environment variables from a JSON file."""
    from ..user_interface.uiux import ui
    try:
        with open(file_path, "r") as f:
            env_config = json.load(f)
            
        # Set environment variables from the config
        for key, value in env_config.items():
            if value:
                os.environ[key] = value
        
        return env_config
    except (FileNotFoundError, json.JSONDecodeError) as e:
        ui.print(f"Error loading config file: {e}", style="danger")
        return None


import sys
from typing import List, Optional
import logging
from .ai_logger import AILogger


_GLOBAL_LOGGER = None

def init_ai_logger(app_name: str, 
                log_file: Optional[str] = None,
                console_output: bool = True,
                log_level: int = logging.WARNING,
                excluded_patterns: Optional[List[str]] = None,
                capture_loggers: Optional[List[str]] = None,
                capture_all_loggers: bool = False,
                use_db: bool = None,
                db_name: Optional[str] = None,
                db_table: Optional[str] = None,
                config_path: Optional[str] = None) -> AILogger:
    """
    Initialize the global AI logger
    
    Args:
        app_name: Name of your application
        log_file: Path to log file (optional)
        console_output: Whether to output to console
        log_level: Logging level (default: WARNING)
        excluded_patterns: List of regex patterns for functions/classes to exclude
        capture_loggers: List of logger names to capture (e.g., ['sqlalchemy', 'uvicorn'])
        capture_all_loggers: If True, capture all loggers (use with caution!)
        use_db: Whether to store logs in database (defaults to config setting)
        db_name: Custom database name (defaults to config)
        db_table: Custom table name for database logging (defaults to app_name)
        config_path: Path to a custom configuration file
        
    Returns:
        AILogger instance
    """
    global _GLOBAL_LOGGER
    
    # If a custom config path is provided, load it
    if config_path:
        from .config import reload_config
        reload_config(config_path)
    
    # Initialize the logger
    _GLOBAL_LOGGER = AILogger(
        app_name=app_name,
        log_file=log_file,
        console_output=console_output,
        log_level=log_level,
        excluded_patterns=excluded_patterns,
        use_db=use_db,
        db_name=db_name,
        db_table=db_table
    )
    
    # Set up global exception handler
    def global_exception_handler(exc_type, exc_value, exc_traceback):
        if _GLOBAL_LOGGER:
            _GLOBAL_LOGGER.log_error(
                error_type=exc_type.__name__,
                error_message=str(exc_value),
                component="global",
                include_traceback=True,
                severity="CRITICAL"
            )
        # Call the default exception handler
        sys.__excepthook__(exc_type, exc_value, exc_traceback)

    sys.excepthook = global_exception_handler
    
    # Capture existing loggers if specified
    if capture_loggers or capture_all_loggers:
        _GLOBAL_LOGGER.capture_existing_loggers(
            logger_names=capture_loggers,
            capture_all=capture_all_loggers
        )
    
    return _GLOBAL_LOGGER

def get_logger() -> AILogger:
    """Get the global AI logger"""
    if _GLOBAL_LOGGER is None:
        raise RuntimeError("Logger not initialized. Call init_ai_logger first.")
    return _GLOBAL_LOGGER

# Auto-wrap decorator
def auto_wrap(func=None, component=None):
    """
    Decorator to automatically wrap functions with the global logger
    
    Usage:
        @auto_wrap
        def my_function():
            pass
            
        @auto_wrap(component="custom_component")
        def another_function():
            pass
    """
    def decorator(f):
        if _GLOBAL_LOGGER is None:
            return f
        return _GLOBAL_LOGGER.wrap_function(component)(f)
        
    if func is None:
        return decorator
    return decorator(func)

# Auto-wrap class decorator
def auto_wrap_class(cls=None, component=None):
    """
    Decorator to automatically wrap classes with the global logger
    
    Usage:
        @auto_wrap_class
        class MyClass:
            pass
            
        @auto_wrap_class(component="custom_component")
        class AnotherClass:
            pass
    """
    def decorator(c):
        if _GLOBAL_LOGGER is None:
            return c
        return _GLOBAL_LOGGER.wrap_class(component)(c)
        
    if cls is None:
        return decorator
    return decorator(cls)


import os
import time
import random
import click
import logging
from pathlib import Path

from ai_logger.src.utils.ai_logger import AILogger

cwd = Path.cwd()
log_file = os.path.join(cwd, "ai_logger_example.json")

logger = AILogger(
    app_name="ai_logger_example",
    log_file=log_file,
    console_output=True,
    log_level=logging.INFO
)

# Example standalone function that will be wrapped
@logger.wrap_function(component="example_module")
def process_data(data_size: int = 100) -> dict:
    """process_data

    Description: Processes a simulated dataset and returns statistics.

    Args:
        data_size (int, optional): Size of the dataset to process. (default: 100).

    Raises:
        ValueError: A value error if the dataset size is less than 100.

    Returns:
        dict: A dictionary containing the processed statistics.
    """
    logger.log_data_event(
        data_source="example_source",
        event_type="data_processing_started",
        record_count=data_size
    )
    
    # Simulate processing time
    time.sleep(0.5)
    
    # Generate fake results
    result = {
        "processed_items": data_size,
        "success_rate": random.uniform(0.90, 0.99),
        "processing_time_ms": random.randint(100, 500)
    }
    
    logger.log_data_event(
        data_source="example_source",
        event_type="data_processing_completed",
        record_count=data_size,
        details=result
    )
    
    # Random chance to throw an error
    if random.random() < 0.1:
        raise ValueError("Random error in data processing")
        
    return result

# Example class that will be wrapped
@logger.wrap_class(component="calculator")
class Calculator:
    def __init__(self, initial_value: float = 0):
        self.value = initial_value
        
    def add(self, x: float) -> float:
        """Add a number to the current value."""
        self.value += x
        return self.value
        
    def subtract(self, x: float) -> float:
        """Subtract a number from the current value."""
        self.value -= x
        return self.value
        
    def multiply(self, x: float) -> float:
        """Multiply the current value by a number."""
        self.value *= x
        return self.value
        
    def divide(self, x: float) -> float:
        """Divide the current value by a number."""
        if x == 0:
            raise ZeroDivisionError("Cannot divide by zero")
        self.value /= x
        return self.value
        
    def complex_calculation(self, x: float, y: float, operation: str) -> float:
        """Perform a more complex calculation."""
        logger.log_model_event(
            model_name="calculator_model",
            event_type="complex_calculation",
            details={"x": x, "y": y, "operation": operation}
        )
        
        if operation == "power":
            result = self.value ** x
        elif operation == "root":
            result = self.value ** (1/x)
        elif operation == "modulo":
            result = self.value % x
        else:
            raise ValueError(f"Unknown operation: {operation}")
            
        self.value = result
        return result

# Module level functions to be wrapped
def module_example():
    """
    Run through and wrap all functions at module level.
    
    This is useful for libraries or utility modules where you want to 
    automatically wrap all functions without decorating each one.
    """
    # Define a simple module-level function to demonstrate wrapping
    @logger.wrap_function(component="module_level")
    def helper_function(x: int, y: int) -> int:
        """A simple helper function to demonstrate module wrapping."""
        return x + y
        
    # Call the helper function we just defined
    result = helper_function(5, 10)
    
    # Log information about the example
    logger.log_data_event(
        data_source="module_example",
        event_type="module_test_completed",
        details={"wrapped_function_result": result}
    )
    
    return result

# Click CLI commands
@click.group()
def cli():
    """AI Logger example application with Click CLI."""

@cli.command()
@click.option("--size", default=100, help="Size of data to process")
def run_function(size: int):
    """Run the wrapped function example."""
    click.echo(f"Running wrapped function example with data size {size}...")
    try:
        result = process_data(size)
        click.echo(f"Processing completed successfully: {result}")
    except Exception as e:
        click.echo(f"Error during processing: {e}")
    click.echo(f"Check the log file at: {log_file}")

@cli.command()
@click.option("--initial", default=10, help="Initial calculator value")
@click.option("--operations", default=5, help="Number of operations to perform")
def run_class(initial: float, operations: int):
    """Run the wrapped class example."""
    click.echo(f"Running calculator class example with initial value {initial}...")
    
    calc = Calculator(initial)
    
    for i in range(operations):
        try:
            operation = random.choice(["add", "subtract", "multiply", "divide", "complex"])
            if operation == "add":
                value = random.uniform(1, 10)
                result = calc.add(value)
                click.echo(f"Add {value}: {result}")
            elif operation == "subtract":
                value = random.uniform(1, 5)
                result = calc.subtract(value)
                click.echo(f"Subtract {value}: {result}")
            elif operation == "multiply":
                value = random.uniform(1, 3)
                result = calc.multiply(value)
                click.echo(f"Multiply by {value}: {result}")
            elif operation == "divide":
                value = random.uniform(0.5, 5)
                result = calc.divide(value)
                click.echo(f"Divide by {value}: {result}")
            elif operation == "complex":
                x = random.uniform(1, 3)
                y = random.uniform(1, 2)
                op = random.choice(["power", "root", "modulo"])
                result = calc.complex_calculation(x, y, op)
                click.echo(f"Complex {op}({x}, {y}): {result}")
        except Exception as e:
            click.echo(f"Error during operation: {e}")
    
    click.echo(f"Final calculator value: {calc.value}")
    click.echo(f"Check the log file at: {log_file}")

@cli.command()
def run_module():
    """Run the module wrapping example."""
    click.echo("Running module wrapping example...")
    
    result = module_example()
    click.echo(f"Module example completed with result: {result}")
    click.echo(f"Check the log file at: {log_file}")

@cli.command()
def run_all():
    """Run all examples in sequence."""
    click.echo("Running all examples...")
    
    click.echo("\n--- Function Example ---")
    run_function.callback(size=150)
    
    click.echo("\n--- Class Example ---")
    run_class.callback(initial=5, operations=7)
    
    click.echo("\n--- Module Example ---")
    run_module.callback()
    
    click.echo("\nAll examples completed!")

if __name__ == "__main__":
    cli()


from pydantic import BaseModel, Field
from typing import Optional, Dict, Any
from datetime import datetime
import uuid

class DatabaseConfig(BaseModel):
    """Configuration for database connection"""
    use_db: bool = False
    dbname: str = "ai_logger_db"
    user: str = "postgres"
    password: str = ""
    host: str = "localhost"
    port: str = "5432"
    table_name: str = "ai_logs"

class LoggingConfig(BaseModel):
    """Configuration for logging settings"""
    level: str = "INFO"
    log_to_file: bool = True
    log_dir: str = "logs"
    log_to_console: bool = False

class AppConfig(BaseModel):
    """Main application configuration"""
    verbose: bool = False
    database: DatabaseConfig = Field(default_factory=DatabaseConfig)
    logging: LoggingConfig = Field(default_factory=LoggingConfig)
    
class AIEvent(BaseModel):
    """Base model for all AI-related events"""
    event_id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    timestamp: datetime = Field(default_factory=datetime.now)
    event_type: str
    component: str
    file_name: Optional[str] = None
    line_number: Optional[int] = None
    function_name: Optional[str] = None
    details: Dict[Any, Any] = {}
    
class ModelEvent(AIEvent):
    """Events specific to model execution"""
    model_name: str
    input_tokens: Optional[int] = None
    output_tokens: Optional[int] = None
    latency_ms: Optional[float] = None
    
class DataEvent(AIEvent):
    """Events related to data processing"""
    data_source: str
    record_count: Optional[int] = None
    
class ErrorEvent(AIEvent):
    """Error events that occur during processing"""
    error_type: str
    error_message: str
    stack_trace: Optional[str] = None
    severity: str = "ERROR"
    
__all__ = [
    "DatabaseConfig",
    "LoggingConfig",
    "AppConfig",
    "AIEvent",
    "ModelEvent",
    "DataEvent",
    "ErrorEvent",
]


from typing import Optional, List
import functools
import json
import logging
import inspect
import traceback
import sys
import os
import time
import re
import pkgutil
import importlib
import psycopg2
from psycopg2.extras import Json
from datetime import datetime
import json as json_module  # Rename to avoid conflict
from pathlib import Path
from .models import AIEvent, ModelEvent, DataEvent, ErrorEvent
from .config import USE_DB, get_db_config

# Custom JSON encoder for datetime objects
class DateTimeEncoder(json_module.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        return super().default(obj)

# Pydantic models for our logging structure


# Main AILogger class
class AILogger:
    def __init__(self, 
                app_name: str,
                log_file: Optional[str] = None,
                console_output: bool = True,
                log_level: int = logging.WARNING,  # Default to WARNING
                excluded_patterns: Optional[List[str]] = None,
                use_db: bool = None,
                db_name: Optional[str] = None,
                db_table: Optional[str] = None):
        """
        Initialize the AI Logger
        
        Args:
            app_name: Name of the application
            log_file: Path to the log file (if None, no file logging)
            console_output: Whether to output to console
            log_level: Logging level (default: WARNING)
            excluded_patterns: List of regex patterns for functions/classes to exclude
            use_db: Whether to store logs in database (overrides config)
            db_name: Custom database name (defaults to config)
            db_table: Custom table name for database logging (defaults to app_name)
        """
        self.app_name = app_name
        self.log_level = log_level
        self.excluded_patterns = excluded_patterns or []
        self._compiled_patterns = [re.compile(pattern) for pattern in self.excluded_patterns]
        self.log_file_path = log_file
        
        # Database configuration
        self.use_db = use_db if use_db is not None else USE_DB
        self.db_config = get_db_config() if self.use_db else None
        
        # Set custom database name if provided
        if db_name and self.db_config:
            self.db_config['dbname'] = db_name
            
        self.db_table = db_table or app_name.lower().replace(' ', '_')
        
        if self.use_db:
            self._init_database()
        
        # Event accumulator - dictionary with timestamps as keys
        self.events = {}
        
        # Set up Python logger for console output
        self.logger = logging.getLogger(app_name)
        self.logger.setLevel(log_level)
        
        # Clear any existing handlers
        if self.logger.hasHandlers():
            self.logger.handlers.clear()
            
        # Create formatter for console output
        formatter = logging.Formatter('%(message)s')
        
        # Console handler
        if console_output:
            console_handler = logging.StreamHandler(sys.stdout)
            console_handler.setFormatter(formatter)
            self.logger.addHandler(console_handler)
            
        # Make sure log directory exists
        if log_file:
            log_dir = os.path.dirname(log_file)
            if log_dir:
                os.makedirs(log_dir, exist_ok=True)
                
            # If log file exists, load existing events
            if os.path.exists(log_file):
                try:
                    with open(log_file, 'r') as f:
                        self.events = json.load(f)
                except (json.JSONDecodeError, FileNotFoundError):
                    # If file is empty or invalid JSON, start with empty dict
                    self.events = {}
            
    def _should_exclude(self, name: str) -> bool:
        """Check if a function or class should be excluded from logging"""
        return any(pattern.search(name) for pattern in self._compiled_patterns)
    
    def _get_caller_info(self, stack_level=2):
        """Get information about the caller"""
        frame = inspect.currentframe()
        # Go up stack_level frames
        for _ in range(stack_level):
            if frame.f_back:
                frame = frame.f_back
            else:
                break
                
        file_name = frame.f_code.co_filename
        line_number = frame.f_lineno
        function_name = frame.f_code.co_name
        
        return {
            "file_name": file_name,
            "line_number": line_number,
            "function_name": function_name
        }
        
    def _create_db_if_not_exists(self):
        """Create database if it doesn't exist"""
        try:
            # Connect to default postgres database to check if our database exists
            conn = psycopg2.connect(
                dbname="postgres",
                user=self.db_config['user'],
                password=self.db_config['password'],
                host=self.db_config['host'],
                port=self.db_config['port']
            )
            conn.autocommit = True  # Enable autocommit for database creation
            cursor = conn.cursor()
            
            # Check if database exists
            cursor.execute("SELECT 1 FROM pg_database WHERE datname=%s", (self.db_config['dbname'],))
            db_exists = cursor.fetchone()
            
            if not db_exists:
                # Database doesn't exist, create it
                cursor.execute(f"CREATE DATABASE {self.db_config['dbname']}")
                print(f"Database '{self.db_config['dbname']}' created successfully")
            else:
                print(f"Database '{self.db_config['dbname']}' already exists")
                
            # Close connection to postgres database
            cursor.close()
            conn.close()
            
        except Exception as e:
            print(f"Error checking/creating database: {e}")
            self.use_db = False
            
    def _init_database(self):
        """Initialize database connection and create table if needed"""
        if not self.use_db:
            return
            
        try:
            # First ensure database exists
            self._create_db_if_not_exists()
            
            # Connect to our database
            self.conn = psycopg2.connect(
                dbname=self.db_config['dbname'],
                user=self.db_config['user'],
                password=self.db_config['password'],
                host=self.db_config['host'],
                port=self.db_config['port']
            )
            self.cursor = self.conn.cursor()
            
            # Create table if it doesn't exist - PostgreSQL syntax
            self.cursor.execute(f'''
            CREATE TABLE IF NOT EXISTS {self.db_table} (
                event_id TEXT PRIMARY KEY,
                timestamp TIMESTAMP,
                event_type TEXT,
                component TEXT,
                file_name TEXT,
                line_number INTEGER,
                function_name TEXT,
                event_json JSONB
            )
            ''')
            
            # Create indexes for common query fields
            self.cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_{self.db_table}_timestamp ON {self.db_table}(timestamp)")
            self.cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_{self.db_table}_event_type ON {self.db_table}(event_type)")
            self.cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_{self.db_table}_component ON {self.db_table}(component)")
            self.conn.commit()
            print(f"Database initialized with table: {self.db_table}")
        except Exception as e:
            print(f"Error initializing database: {e}")
            self.use_db = False
    
    def _log_to_database(self, event_dict):
        """Log event to database"""
        if not self.use_db:
            return
            
        try:
            # Prepare values for INSERT
            event_id = event_dict["event_id"]
            timestamp = event_dict["timestamp"].isoformat() if isinstance(event_dict["timestamp"], datetime) else event_dict["timestamp"]
            event_type = event_dict["event_type"]
            component = event_dict["component"]
            file_name = event_dict["file_name"]
            line_number = event_dict["line_number"]
            function_name = event_dict["function_name"]
            # First convert datetime objects to strings using our custom encoder
            serialized_dict = json_module.loads(json_module.dumps(event_dict, cls=DateTimeEncoder))
            
            # Use psycopg2's Json adapter for proper JSONB handling
            event_json = Json(serialized_dict)
            
            # Insert into database (PostgreSQL uses %s for parameters)
            self.cursor.execute(
                f"""
                INSERT INTO {self.db_table} 
                (event_id, timestamp, event_type, component, file_name, line_number, function_name, event_json)
                VALUES (%s, %s, %s, %s, %s, %s, %s, %s)
                ON CONFLICT (event_id) DO UPDATE SET 
                    timestamp = EXCLUDED.timestamp,
                    event_type = EXCLUDED.event_type,
                    component = EXCLUDED.component,
                    file_name = EXCLUDED.file_name,
                    line_number = EXCLUDED.line_number,
                    function_name = EXCLUDED.function_name,
                    event_json = EXCLUDED.event_json
                """,
                (event_id, timestamp, event_type, component, file_name, line_number, function_name, event_json)
            )
            self.conn.commit()
        except Exception as e:
            print(f"Error logging to database: {e}")
    
    def _log_event(self, event: AIEvent):
        """
        Add event to the event dictionary and update log file
        Uses timestamp as key and event as value in the dictionary
        """
        # Add application info
        event.details.update({
            "app_name": self.app_name,
        })
        
        # Convert to dict for storage
        event_dict = event.model_dump()
        
        # Use the timestamp as the key (formatted as ISO string)
        timestamp_key = event_dict["timestamp"].isoformat()
        
        # Store the event in our dictionary
        self.events[timestamp_key] = event_dict
        
        # If this is above log level, also output to console
        if self.logger.level <= self.log_level:
            self.logger.log(self.log_level, event.model_dump_json())
        
        # Write to file if file logging is enabled
        if self.log_file_path:
            # Write the entire events dictionary to the file with indentation
            with open(self.log_file_path, 'w') as f:
                json_module.dump(self.events, f, indent=2, cls=DateTimeEncoder)
                
        # Log to database if enabled
        if self.use_db:
            self._log_to_database(event_dict)
        
    def log_model_event(self, model_name: str, event_type: str, 
                        input_tokens: Optional[int] = None,
                        output_tokens: Optional[int] = None,
                        latency_ms: Optional[float] = None,
                        **details):
        """Log a model-related event with caller info"""
        caller_info = self._get_caller_info()
        
        event = ModelEvent(
            event_type=event_type,
            component="model",
            model_name=model_name,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
            latency_ms=latency_ms,
            file_name=caller_info["file_name"],
            line_number=caller_info["line_number"],
            function_name=caller_info["function_name"],
            details=details
        )
        self._log_event(event)
        
    def log_data_event(self, data_source: str, event_type: str,
                      record_count: Optional[int] = None,
                      **details):
        """Log a data-related event with caller info"""
        caller_info = self._get_caller_info()
        
        event = DataEvent(
            event_type=event_type,
            component="data",
            data_source=data_source,
            record_count=record_count,
            file_name=caller_info["file_name"],
            line_number=caller_info["line_number"],
            function_name=caller_info["function_name"],
            details=details
        )
        self._log_event(event)
        
    def log_error(self, error_type: str, error_message: str,
                 component: str = "application",
                 include_traceback: bool = True,
                 severity: str = "ERROR"):
        """Log an error event with caller info"""
        caller_info = self._get_caller_info()
        
        stack_trace = None
        if include_traceback:
            stack_trace = traceback.format_exc()
            
        event = ErrorEvent(
            event_type="error",
            component=component,
            error_type=error_type,
            error_message=error_message,
            stack_trace=stack_trace,
            severity=severity,
            file_name=caller_info["file_name"],
            line_number=caller_info["line_number"],
            function_name=caller_info["function_name"]
        )
        self._log_event(event)
        
    def wrap_function(self, component: str = None):
        """
        Decorator to wrap functions and log their execution
        
        Args:
            component: The component name (if None, uses function module)
        """
        def decorator(func):
            # Check if function should be excluded
            if self._should_exclude(func.__name__):
                return func
                
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                # Get component name
                comp = component
                if comp is None:
                    comp = func.__module__ or "unknown"
                
                # Get caller info
                caller_info = self._get_caller_info(stack_level=1)
                
                # Log the start
                start_time = time.time()
                
                try:
                    # Execute the function
                    result = func(*args, **kwargs)
                    
                    # Calculate duration
                    (time.time() - start_time) * 1000
                    
                    # Return result without logging success if below ERROR level
                    return result
                    
                except Exception as e:
                    # Always log errors
                    self.log_error(
                        error_type=type(e).__name__,
                        error_message=str(e),
                        component=comp
                    )
                    raise
                    
            return wrapper
        return decorator
    
    def wrap_class(self, component: str = None):
        """
        Class decorator to wrap all methods of a class
        
        Args:
            component: The component name (if None, uses class name)
        """
        def decorator(cls):
            # Check if class should be excluded
            if self._should_exclude(cls.__name__):
                return cls
                
            comp = component
            if comp is None:
                comp = cls.__name__
                
            # Find all methods that aren't special methods
            for name, method in inspect.getmembers(cls, inspect.isfunction):
                if not name.startswith('__') and not self._should_exclude(name):
                    setattr(cls, name, self.wrap_function(comp)(method))
            return cls
        return decorator

    def wrap_all_modules(self, package_paths: List[str], exclude_modules: Optional[List[str]] = None):
        """
        Wrap all functions and classes in specified modules
        
        Args:
            package_paths: List of package paths to wrap
            exclude_modules: List of module names to exclude
        """
        exclude_modules = exclude_modules or []
        
        for package_path in package_paths:
            package = __import__(package_path, fromlist=['*'])
            for module_info in pkgutil.iter_modules([os.path.dirname(package.__file__)]):
                module_name = f"{package_path}.{module_info.name}"
                
                # Skip excluded modules
                if module_name in exclude_modules or any(self._should_exclude(name) for name in [module_name, module_info.name]):
                    continue
                
                try:
                    module = importlib.import_module(module_name)
                    
                    # Wrap all functions
                    for name, obj in inspect.getmembers(module, inspect.isfunction):
                        if obj.__module__ == module_name and not self._should_exclude(name):
                            setattr(module, name, self.wrap_function(module_name)(obj))
                    
                    # Wrap all classes
                    for name, obj in inspect.getmembers(module, inspect.isclass):
                        if obj.__module__ == module_name and not self._should_exclude(name):
                            setattr(module, name, self.wrap_class(module_name)(obj))
                            
                except (ImportError, AttributeError) as e:
                    print(f"Error wrapping module {module_name}: {e}")

    def __del__(self):
        """Destructor to close database connection when logger is destroyed"""
        if hasattr(self, 'conn') and self.conn is not None:
            try:
                self.conn.close()
                print("Database connection closed")
            except:
                pass
    
    def capture_existing_loggers(self, logger_names=None, capture_all=False, min_level=None):
        """
        Redirect logs from existing loggers to this AI logger
        
        Args:
            logger_names: List of logger names to capture (e.g., ['sqlalchemy', 'uvicorn'])
            capture_all: If True, capture all loggers (use with caution!)
            min_level: Minimum level to capture (defaults to self.log_level)
        """
        if logger_names is None and not capture_all:
            logger_names = []
        
        # Use provided min_level or default to this logger's level
        capture_level = min_level if min_level is not None else self.log_level
        
        # Custom handler to redirect logs to our logger
        class AILogHandler(logging.Handler):
            def __init__(self, ai_logger, min_capture_level):
                super().__init__()
                self.ai_logger = ai_logger
                # Set the handler level to filter logs before they reach emit
                self.setLevel(min_capture_level)
                
            def emit(self, record):
                try:
                    # The handler level should filter, but double-check just in case
                    if record.levelno < self.level:
                        return
                        
                    # Map the log level to severity
                    severity_map = {
                        logging.DEBUG: "DEBUG",
                        logging.INFO: "INFO", 
                        logging.WARNING: "WARNING",
                        logging.ERROR: "ERROR",
                        logging.CRITICAL: "CRITICAL"
                    }
                    severity = severity_map.get(record.levelno, "INFO")
                    
                    # Get the log message
                    msg = self.format(record)
                    
                    # Avoid recursive logging - don't log messages from our own logger
                    if record.name == self.ai_logger.app_name:
                        return
                        
                    # Create event directly to avoid triggering more log messages
                    event = ErrorEvent(
                        event_type="error",
                        component=record.name,
                        error_type="LogMessage",
                        error_message=msg,
                        stack_trace=None,
                        severity=severity,
                        file_name=getattr(record, 'pathname', None),
                        line_number=getattr(record, 'lineno', None),
                        function_name=getattr(record, 'funcName', None),
                        details={"logger_name": record.name}
                    )
                    
                    # Add app name to details
                    event.details.update({
                        "app_name": self.ai_logger.app_name,
                    })
                    
                    # Convert to dict for storage
                    event_dict = event.model_dump()
                    
                    # Use the timestamp as the key (formatted as ISO string)
                    timestamp_key = event_dict["timestamp"].isoformat()
                    
                    # Store the event in our dictionary without triggering logging
                    self.ai_logger.events[timestamp_key] = event_dict
                    
                    # Write to file directly without going through logger
                    if self.ai_logger.log_file_path:
                        with open(self.ai_logger.log_file_path, 'w') as f:
                            json_module.dump(self.ai_logger.events, f, indent=2, cls=DateTimeEncoder)
                            
                except Exception:
                    self.handleError(record)
        
        # Create the handler instance with our capture level
        ai_handler = AILogHandler(self, capture_level)
        
        # Set formatter for the handler
        formatter = logging.Formatter('%(message)s')
        ai_handler.setFormatter(formatter)
        
        if capture_all:
            # Get the root logger to capture all logs
            root_logger = logging.getLogger()
            root_logger.addHandler(ai_handler)
            print(f"Capturing all logs at level {logging.getLevelName(capture_level)} and above")
            return
            
        # Add the handler to each specified logger
        for logger_name in logger_names:
            try:
                logger = logging.getLogger(logger_name)
                logger.addHandler(ai_handler)
                print(f"Capturing logs from '{logger_name}' at level {logging.getLevelName(capture_level)} and above")
            except Exception as e:
                print(f"Error capturing logger '{logger_name}': {e}")
                    
# Global instance and setup functions



import json
from rich.console import Console, ConsoleOptions, RenderResult
from rich.live import Live
from rich.markdown import CodeBlock, Markdown
from rich.syntax import Syntax
from rich.text import Text
from rich.theme import Theme
from rich.panel import Panel
from rich.markup import escape
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeElapsedColumn
from rich.status import Status
from contextlib import contextmanager

class CyberpunkUI:
    def __init__(self):
        self.theme, self.colors = self._create_cyberpunk_theme()
        self._setup_prettier_code_blocks()
        self.console = Console(theme=self.theme, emoji=True, emoji_variant="emoji")
        self.live = Live(console=self.console, auto_refresh=False)
        self.live_active = False
        self.last_content = None  # Store the last displayed content
    
    def _create_cyberpunk_theme(self):
        # Cyberpunk color palette
        cyberpunk_colors = {
            "neon_pink": "#FF00FF",
            "electric_blue": "#00FFFF",
            "acid_green": "#39FF14",
            "cyber_yellow": "#FFD300",
            "digital_purple": "#9A00FF",
            "terminal_green": "#00FF41",
            "hacker_red": "#FF3131"
        }

        # Create cyberpunk theme
        cyberpunk_theme = Theme({
            "info": f"bold {cyberpunk_colors['terminal_green']}",
            "warning": f"bold {cyberpunk_colors['cyber_yellow']}",
            "danger": f"bold {cyberpunk_colors['hacker_red']}",
            "prompt": f"bold {cyberpunk_colors['neon_pink']}",
            "header": f"bold {cyberpunk_colors['electric_blue']}",
            "success": f"bold {cyberpunk_colors['acid_green']}",
            "highlight": f"bold {cyberpunk_colors['digital_purple']}",
            "default": "white",
        })
        return cyberpunk_theme, cyberpunk_colors
    
    def _setup_prettier_code_blocks(self):
        """Make rich code blocks prettier and easier to copy."""
        class SimpleCodeBlock(CodeBlock):
            def __rich_console__(
                self, console: Console, options: ConsoleOptions
            ) -> RenderResult:
                code = str(self.text).rstrip()
                yield Text(self.lexer_name, style='dim')
                yield Syntax(
                    code,
                    self.lexer_name,
                    theme=self.theme,
                    background_color='default',
                    word_wrap=True,
                )
                yield Text(f'/{self.lexer_name}', style='dim')

        Markdown.elements['fence'] = SimpleCodeBlock
    
    def start_live(self):
        """Start the live display if not already active"""
        if not self.live_active:
            self.live.start()
            self.live_active = True
            # Restore previous content if available
            if self.last_content:
                self.live.update(self.last_content)
                self.live.refresh()
    
    def stop_live(self):
        """Stop the live display if active"""
        if self.live_active:
            self.live.stop()
            self.live_active = False
    
    def print(self, message, style="info"):
        """Print with cyberpunk styling"""
        formatted_message = f"[{style}]{message}[/{style}]"
        self.last_content = formatted_message  # Store content for restoration
        
        if self.live_active:
            self.live.update(formatted_message)
            self.live.refresh()
        else:
            self.console.print(formatted_message)
    
    def display_json(self, data, title=None):
        """Safely display JSON data in cyberpunk style"""
        json_theme = "dracula"
        
        # Process title if provided
        if title:
            title_panel = Panel(f"[header]>>> {title.upper()} <<<[/header]")
            self.last_content = title_panel  # Store content for restoration
            
            if self.live_active:
                self.live.update(title_panel)
                self.live.refresh()
            else:
                self.console.print(title_panel)
        
        # Check if data is already a string
        panel = None  # Initialize panel variable
        
        if isinstance(data, str):
            try:
                # Try to parse to ensure it's valid JSON
                panel = Panel(Syntax(data, "json", theme=json_theme, background_color="default"))
            except json.JSONDecodeError:
                try:
                    # Try to parse and then format as JSON
                    parsed_data = json.loads(data)
                    json_str = json.dumps(parsed_data, indent=4)
                    panel = Panel(Syntax(json_str, "json", theme=json_theme, background_color="default"))
                except json.JSONDecodeError:
                    # If not valid JSON, display as text
                    panel = Panel(Text(data))
        else:
            # Convert Python object to JSON string
            json_str = json.dumps(data, indent=4)
            panel = Panel(Syntax(json_str, "json", theme=json_theme, background_color="default"))
        
        # Store and display the panel if created
        if panel:
            self.last_content = panel  # Store content for restoration
            if self.live_active:
                self.live.update(panel)
                self.live.refresh()
            else:
                self.console.print(panel)
    
    def display_header(self, title, subtitle=None):
        """Display a cyberpunk-styled header"""
        header_text = f"[header]>>> {title.upper()} <<<[/header]"
        panel = None  # Initialize panel variable
        
        if subtitle:
            if isinstance(subtitle, (dict, list)):
                # Display header and then JSON separately
                panel = Panel(header_text, border_style=self.colors["neon_pink"])
                self.last_content = panel  # Store content for restoration
                
                if self.live_active:
                    self.live.update(panel)
                    self.live.refresh()
                else:
                    self.console.print(panel)
                self.display_json(subtitle)
                return
            elif isinstance(subtitle, str) and (subtitle.startswith("{") or subtitle.startswith("[")):
                # Handle JSON strings
                panel = Panel(header_text, border_style=self.colors["neon_pink"])
                self.last_content = panel  # Store content for restoration
                
                if self.live_active:
                    self.live.update(panel)
                    self.live.refresh()
                else:
                    self.console.print(panel)
                try:
                    # Try to parse and display as JSON
                    self.display_json(subtitle)
                    return
                except json.JSONDecodeError:
                    # If not valid JSON, display as escaped text
                    json_panel = Panel(Text(subtitle), border_style=self.colors["highlight"])
                    self.last_content = json_panel  # Store content for restoration
                    
                    if self.live_active:
                        self.live.update(json_panel)
                        self.live.refresh()
                    else:
                        self.console.print(json_panel)
                    return
            else:
                header_text += f"\n[highlight]{escape(str(subtitle))}[/highlight]"
        
        panel = Panel(header_text, border_style=self.colors["neon_pink"])
        self.last_content = panel  # Store content for restoration
        
        if self.live_active:
            self.live.update(panel)
            self.live.refresh()
        else:
            self.console.print(panel)
    
    @contextmanager
    def spinner(self, message: str, spinner_type: str = None):
        """
        Display a spinner with a message during a long-running operation.
        
        Args:
            message: The message to display alongside the spinner
            spinner_type: The type of spinner to display (dots, dots2, dots3, etc.)
        """
        # Import here to avoid circular import
        from config import CONFIG, get_spinner_type
        
        # Use provided spinner type or get from config
        if spinner_type is None:
            spinner_type = get_spinner_type()
        
        # Check if loading animations are enabled in config
        if not CONFIG.ui.loading_animation:
            # Simple non-animation fallback with no output
            try:
                yield None
            finally:
                pass
            return
        
        spinner_style = self.colors["electric_blue"]
        message_style = self.colors["terminal_green"]
        
        # Save the current content before stopping live
        saved_content = self.last_content
        
        # Use progress bar style for AI agent operations
        if "AI agent" in message or "agent" in message.lower():
            # Make sure any Live display is stopped before starting Progress
            was_live_active = self.live_active
            if was_live_active:
                self.stop_live()
                
            with Progress(
                TextColumn(f"[{message_style}]{message}[/{message_style}]"),
                BarColumn(complete_style=self.colors["acid_green"]),
                console=self.console
            ) as progress:
                task = progress.add_task("", total=None)
                try:
                    yield progress
                finally:
                    progress.stop()
                    # Add a blank line after the progress bar is done
                    self.console.print()
                    
                    # Restore the saved content and restart Live if it was active before
                    if saved_content:
                        self.last_content = saved_content
                        
                    if was_live_active:
                        self.start_live()
        else:
            # Use spinner for other operations
            # Make sure any Live display is stopped before starting Status
            was_live_active = self.live_active
            if was_live_active:
                self.stop_live()
                
            with Status(f"[{spinner_style}][[/{spinner_style}][{message_style}]{message}[/{message_style}][{spinner_style}]][/{spinner_style}]", 
                    spinner=spinner_type, spinner_style=spinner_style, console=self.console) as status:
                try:
                    yield status
                finally:
                    status.stop()
                    # Add a blank line after the spinner is done
                    self.console.print()
                    
                    # Restore the saved content and restart Live if it was active before
                    if saved_content:
                        self.last_content = saved_content
                        
                    if was_live_active:
                        self.start_live()
    
    @contextmanager
    def progress(self, total: int, description: str = "Processing"):
        """
        Display a progress bar for a task with known total steps.
        
        Args:
            total: The total number of steps to complete
            description: Description of the task being performed
        """
        # Import here to avoid circular import
        from config import CONFIG, get_spinner_type
        
        # Save the current content before potentially stopping live
        saved_content = self.last_content
        
        # Check if loading animations are enabled in config
        if not CONFIG.ui.loading_animation:
            # Simple non-animation fallback
            progress_message = f"[info]{description} (0/{total})[/info]"
            self.last_content = progress_message  # Store content for restoration
            
            if self.live_active:
                self.live.update(progress_message)
                self.live.refresh()
            else:
                self.console.print(progress_message)
            
            count = 0
            
            def update(n=1):
                nonlocal count
                count += n
                if count % max(1, total // 10) == 0 or count >= total:  # Print status at 10% intervals
                    current_message = f"[info]{description} ({count}/{total})[/info]"
                    self.last_content = current_message  # Store content for restoration
                    
                    if self.live_active:
                        self.live.update(current_message)
                        self.live.refresh()
                    else:
                        self.console.print(current_message)
            
            try:
                yield update
            finally:
                final_message = f"[info]{description} completed ({total}/{total})[/info]"
                self.last_content = final_message  # Store content for restoration
                
                if self.live_active:
                    self.live.update(final_message)
                    self.live.refresh()
                else:
                    self.console.print(final_message)
            return
        
        progress_style = self.colors["acid_green"]
        desc_style = self.colors["terminal_green"]
        time_style = self.colors["cyber_yellow"]
        spinner_type = get_spinner_type()
        
        # Make sure any Live display is stopped before starting Progress
        was_live_active = self.live_active
        if was_live_active:
            self.stop_live()
            
        with Progress(
            SpinnerColumn(spinner_name=spinner_type, style=self.colors["electric_blue"]),
            TextColumn(f"[{desc_style}]{description}[/{desc_style}]"),
            BarColumn(complete_style=progress_style),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
            TimeElapsedColumn(),
            console=self.console
        ) as progress:
            task = progress.add_task(description, total=total)
            try:
                yield lambda n=1: progress.update(task, advance=n)
            finally:
                # Restore the saved content and restart Live if it was active before
                if saved_content:
                    self.last_content = saved_content
                    
                if was_live_active:
                    self.start_live()
    
    def indeterminate_progress(self, description: str = "Processing..."):
        """Create an indeterminate progress bar for tasks with unknown duration."""
        # Import here to avoid circular import
        from config import CONFIG, get_spinner_type
        
        # Check if loading animations are enabled
        if not CONFIG.ui.loading_animation:
            progress_message = f"[info]{description}[/info]"
            self.last_content = progress_message  # Store content for restoration
            
            if self.live_active:
                self.live.update(progress_message)
                self.live.refresh()
            else:
                self.console.print(progress_message)
            return None
        
        spinner_type = get_spinner_type()
        
        return Progress(
            SpinnerColumn(spinner_name=spinner_type, style=self.colors["electric_blue"]),
            TextColumn(f"[{self.colors['terminal_green']}]{description}[/]"),
            console=self.console
        )


# Initialize the UI
ui = CyberpunkUI()

__all__ = ["ui"]


from .ai_logger import AILogger
from .logger_tools import init_ai_logger, auto_wrap, auto_wrap_class
from .models import (
    DatabaseConfig,
    LoggingConfig,
    AppConfig,
    AIEvent,
    ModelEvent,
    DataEvent,
    ErrorEvent
)
from .config import CONFIG, reload_config, get_db_config

__all__ = [
    "AILogger",
    "init_ai_logger",
    "auto_wrap",
    "auto_wrap_class",
    "DatabaseConfig",
    "LoggingConfig",
    "AppConfig",
    "AIEvent",
    "ModelEvent",
    "DataEvent",
    "ErrorEvent",
    "CONFIG",
    "reload_config",
    "get_db_config",
]


from __future__ import annotations
from ai_logger.src.user_interface import ui
from ai_logger.src.utils import (
    # Configuration variables
    CONFIG,
    reload_config,
    get_db_config,
    
    # Important models
    AppConfig,
    AIEvent,
    ModelEvent,
    DataEvent,
    ErrorEvent,
    LoggingConfig,
    DatabaseConfig,
    
    # Logger tools
    AILogger,
    init_ai_logger,
    auto_wrap,
    auto_wrap_class,
)

__all__ = [
    "ui",
    "CONFIG",
    "reload_config",
    "get_db_config",
    "AILogger",
    "init_ai_logger",
    "auto_wrap",
    "auto_wrap_class",
    "DatabaseConfig",
    "LoggingConfig",
    "AppConfig",
    "AIEvent",
    "ModelEvent",
    "DataEvent",
    "ErrorEvent",
]